<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage adding or editing forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * Page object to handle the adding or editing forms associated with a projection to the database.
 * 
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageFormParentProjection extends I2CE_PageForm {
        
    /**
     * Return the form name for this page.
     * 
     * It will be used for the default form template and php page for the form submission.
     * @param boolean $html Set to true if this is to be used for the html template page to load.
     * @return string
     */
    protected function getForm( $html=false ) { return $this->form_name; }

    /**
     * @var integer The record id number of the object being edited.
     */
    protected $id;
    /**
     * @var integer The recored if number of the parent of the object being edited
     */
    protected $parent_id;
    /**
     * The form name being edited by this page.
     * @var string
     */
    protected $form_name;

    /**
     * The link used to access this form
     * $var protected string $form_link
     */
    protected $form_link;
        
    /**
     * Sets the form link 
     * @param string $link
     */
    public function setFormLink($link) {
        $this->form_link = $link;
    }
        

    protected function getTitle() {
        $title = '';
        $path = '/' . $this->module() . '/page/' . $this->page() . '/args/title';
        I2CE::getConfig()->__setIfIsSet($title,$path);
        return $title;
    }


    /**
     * Create a new instance of this page.
     * 
     * This will call the parent constructor and then setup the base
     * template pages for the {@link I2CE_Template template}.  It also sets up the values
     * for the member variables.
     * @param array $args
     * @param string $request_remainder
     */
    public function __construct( $args,$request_remainder){
        parent::__construct($args,$request_remainder);
        $form_name = $args['page_form'];
        if(empty($form_name)) {
            I2CE::raiseError("No form name specified", E_USER_ERROR);
        }
        $this->form_name = $form_name;
        $this->form_link = $form_name ;
        $this->id = 0;
        if ( $this->isPost() && $this->post_exists( 'id' ) ) {
            $this->id = $this->post('id');
        } elseif ( $this->get_exists( 'id' ) ) {
            $this->id = $this->get('id');
        }
        if ( $this->isPost() && $this->post_exists( 'parent' ) ) {
            $this->parent_id = $this->post('parent');
        } elseif ( $this->get_exists( 'parent' ) ) {
            $this->parent_id = $this->get('parent');
        }
    }
        
    /**
     * Load the HTML template files for editing.
     */
    protected function loadHTMLTemplates() {
        parent::loadHTMLTemplates();
        $this->template->appendFileById( "menu_view_link.html", "li", "navBarUL", true );
        $this->template->appendFileById( "form_" . $this->getForm( true ) . ".html", "tbody", "projection_form" );
        $this->template->addFile( "instruct_" . $this->getForm( true ) . ".html", "div" );
    }
                
    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.  It determines the type based on the
     * {@link $type} member variable.
     */
    protected function loadObjects() {
        if ( $this->factory->exists( $this->getForm() ) )
            $this->setObject( $this->factory->createForm( $this->getForm(), $this->id ) );
        $parent = $this->factory->createForm( "projection", $this->parent_id );
        $parent->populate();
        $this->setObject( $parent, I2CE_PageForm::EDIT_PARENT );
        parent::loadObjects();
        /*
                $this->edit_obj->setParent( $this->parent->getId() );
                if ( $this->isPost() ) {
                        $this->edit_obj->load( $_POST );
                } elseif ( $this->edit_obj->getId() > 0 ) {
                        $this->edit_obj->populate();
                        $this->setEditing();
                }
        */
    }
        
    /**
     * Set the data to be displayed for the page.
     */
    protected function setDisplayData() {
        parent::setDisplayData();
        $this->template->setDisplayData( "projection_header", $this->getTitle() );
        $this->template->setDisplayData( "projection_form", $this->form_link);
    }


    /**
     * Save the objects to the database.
     * 
     * Save the default object being edited and return to the view page.
     * @global array
     */
    protected function save() {
        parent::save();
        $this->setRedirect(  "view?id=" . $this->getPrimary()->getParent() );
    }
                
}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
