<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a projection report.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a projection report.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageProjectionReport extends I2CE_Page { 
    /**
     * @var iHRIS_Projection The projection form used for this report.
     */
    protected $projection;
        
    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        $factory = I2CE_FormFactory::instance();
        $this->projection = $factory->createForm( "projection", $this->get('parent') );
        $this->projection->populate();
        $this->projection->populateChildren( array( "pool" ) );

        $ids = array();
        if ( $this->get_exists( 'id' ) ) {
            if ( is_array( $this->get('id') ) ) {
                $ids = $this->get('id');
            } else {
                $ids[] = $this->get('id');
            }
        }
        $includes = null;
        if ( $this->post_exists( 'check_includes' ) ) {
            $includes = $this->post("include");
            if ( $includes == null ) $includes = array();
        } elseif ( $this->get_exists( 'check_includes' ) ) {
            $includes = $this->get("include");
            if ( $includes == null ) $includes = array();
        }

        if ( $this->get_exists( 'data' ) ) {
            $population = $this->projection->projectPopulation();
            $required = array( "" => "Required" );
            $supply = array( "" => "Supply" );
            $cost = array( "" => "Total Cost" );
            $orig = array( "" => "Supply (Original)" );
            $orig_cost = array( "" => "Total Cost (Original)" );
            $init_required = array( "" => "");
            $init_supply = array( "" => "");
            $breakdown = array();
            $axis = array_merge( array( "" ), array_keys( $population['pop'] ) );
            $use_initial = false;
            foreach( $this->projection->children['pool'] as $pool ) {
                if ( in_array( $pool->getId(), $ids ) ) {
                    /*
                    $breakdown[ $pool->getId() ] = array( 
                            'required' => array( "" => $pool->name . " Required" ),
                            'supply' => array( "" => $pool->name . " Supply" ),
                            'orig' => array( "" => $pool->name . " Supply (Original)" ),
                            'cost' => array( "" => $pool->name . " Cost" ),
                            'orig_cost' => array( "" => $pool->name . " Cost (Original)" ),
                            );
                            */
                    $pool_data = $pool->projectPool( $includes );
                    $orig_data = $pool->projectPool();
                    $use_pool_initial = false;
                    if ( count( $pool_data['real']['begin_supply'] ) > 1 || count( $pool_data['real']['target'] ) + count( $pool_data['real']['ratio'] ) > 1 ) {
                        $years = array_keys( $population['pop'] );
                        $initial_data = $pool->projectPool( $includes, $years[0] );
                        $use_pool_initial = true;
                        $use_initial = true;
                        $init_required[""] = "Required (" . $years[0] . ")";
                        $init_supply[""] = "Supply (" . $years[0] . ")";
                    }
                    foreach( $population['pop'] as $year => $pop ) {
                        $required[$year] += $pool_data['target'][$year];
                        $supply[$year] += $pool_data['begin_supply'][$year];
                        $orig[$year] += $orig_data['begin_supply'][$year];
                        $year_cost = $pool_data['total_cost'][$year]->getValue();
                        $cost[$year] += $year_cost[1];
                        $cost_title = I2CE_List::lookup( $year_cost[0], 'currency' );
                        $orig_year_cost = $orig_data['total_cost'][$year]->getValue();
                        $orig_cost[$year] += $orig_year_cost[1];
                        if ( $use_pool_initial ) {
                            $init_required[$year] += $initial_data['target'][$year];
                            $init_supply[$year] += $initial_data['begin_supply'][$year];
                        } else {
                            $init_required[$year] += $pool_data['target'][$year];
                            $init_supply[$year] += $pool_data['begin_supply'][$year];
                        }
                        

                        /*
                        $breakdown[$pool->getId()]['required'][$year] = $pool_data['target'][$year];
                        $breakdown[$pool->getId()]['supply'][$year] = $pool_data['begin_supply'][$year];
                        $breakdown[$pool->getId()]['orig'][$year] = $orig_data['begin_supply'][$year];
                        $breakdown[$pool->getId()]['cost'][$year] = $year_cost[1];
                        $breakdown[$pool->getId()]['orig_cost'][$year] = $orig_year_cost[1];
                        */
                    }
                }
            }
            require_once( I2CE::getFileSearch()->search( 'MAANI_CHART_FILES', 'charts.php'));
            $chart = array ( 
                'chart_type' => "line",
                'chart_value' => array( 'position' => "cursor", 'size' => 10, 'separator' => ',', 'color' => 'dddddd', 'background_color' => '333333' ),
                'chart_pref' => array( 'line_thickness' => 2, "point_shape" => "none" ),
                'axis_value' => array( 'size' => 12 ),
                'legend_label' => array( 'size' => 12 ),
                'axis_ticks' => array( 'major_thickness' => 1, 'major_color' => "222222", 
                    'minor_thickness' => 1, 'minor_color' => "aaaaaa", 'minor_count' => 3,
                    'value_ticks' => true, 'category_ticks' => true, 'position' => "centered" ),
                'chart_grid_v' => array( 'thickness' => 1, 'color' => '333333', 'type' => "dotted" ),
                );
            if ( $this->get_exists( 'cost' ) ) {
                $chart['chart_data'] = array( $axis, $cost );
                if ( count( array_diff_assoc( $cost, $orig_cost ) ) > 1 ) {
                    $chart['chart_data'][1][""] .= " (Modified)";
                    $chart['chart_data'][] = $orig_cost;
                }
                $chart['series_color'] = array( "69808f", "06a349" );
                $chart['axis_category']["size"] = 8;
                $chart['legend_label']["size"] = 8;
                $chart['axis_value']["size"] = 8;
            } else {
                /*
                if ( count( $breakdown ) > 1 ) {
                    $chart['legend_label']["size"] = 8;
                    $chart['chart_data'][] = $axis;
                    $chart['chart_type'] = array();
                    $chart['series_color'] = array();
                    $colors = array( "a30649", "69808f", "06a349" );
                    foreach ( $breakdown as $pool => $data ) {
                        $chart['chart_type'][] = "line";
                        $chart['chart_type'][] = "line";
                        $chart['chart_data'][] = $data['required'];
                        $chart['chart_data'][] = $data['supply'];
                        $color = current( $colors );
                        $chart['series_color'][] = $color;
                        $chart['series_color'][] = self::darken( $color, 10 );
                        if ( count( array_diff_assoc( $supply, $orig ) ) > 1 ) {
                            $chart['chart_data'][count($chart['chart_data'])-1][""] .= " (Modified)";
                            $chart['chart_data'][] = $data['orig'];
                            $chart['chart_type'][] = "line";
                            $chart['series_color'][] = self::darken( $color, 20 );
                        }
                        if ( next( $colors ) == null ) {
                            reset( $colors );
                        }
                    }
                } else {
                */
                    $chart['chart_data'] = array( $axis );
                    if ( $use_initial ) {
                        $chart['chart_data'][] = $init_required;
                        $chart['chart_data'][] = $init_supply;
                        $chart['series_color'][] = "480220";
                        $chart['series_color'][] = "262f34";
                    }
                    $chart['chart_data'][] = $required;
                    $chart['chart_data'][] = $supply;
                    $chart['series_color'][] = "a30649";
                    $chart['series_color'][] = "69808f";
                    if ( count( array_diff_assoc( $supply, $orig ) ) > 1 ) {
                        $chart['chart_data'][2][""] .= " (Modified)";
                        if ( $use_initial ) {
                            $chart['chart_data'][4][""] .= " (Modified)";
                        }
                        $chart['chart_data'][] = $orig;
                        $chart['series_color'][] = "06a349";
                    }
                /* } */
                $chart['axis_category'] = array( "size" => 10, "orientation" => "diagonal_down" );
            }
            if ( $this->get_exists( 'cost' ) ) {
                $chart['draw'][] = array( 'type' => 'text', 'rotation' => -90, 'size' => 20, 
                        'font' => 'arial', 'color' => "003300", 'alpha' => '50',
                        'x' => 0, 'y' => 100, 'width' => 90, 'height' => 30,
                        'text' => $cost_title, 'h_align' => 'center' );
            } elseif ( $this->get_exists( 'title' ) ) {
                $chart['draw'][] = array( 'type' => 'text', 'rotation' => -90, 'size' => 30, 
                        'font' => 'arial', 'color' => "000000", 'alpha' => '50',
                        'x' => -5, 'y' => 200, 'width' => 170, 'height' => 60,
                        'text' => $this->get('title'), 'h_align' => 'center' );
            }
            SendChartData( $chart );
            exit();
        }  else {
            parent::action();

            $this->template->appendFileById( "menu_view_link.html", "li", "navBarUL", true );
            $this->template->appendFileById( "menu_view_projection.html", "ul", "menuView" );
            $this->template->setAttribute( "class", "active", "menu_report_link" );
            $this->template->setForm( $this->projection );
    
            $pools = array();
            foreach( $this->projection->children['pool'] as $pool ) {
                $node = $this->template->appendFileById( "menu_view_pool.html", "li", "menuViewProjection" );
                $this->template->setForm( $pool, $node );
                $this->template->addOption( "pool_list", $pool->getId(), $pool->name );
                if ( in_array( $pool->getId(), $ids ) ) {
                    $pools[ $pool->getId() ] = $pool;
                }
            }
            if ( count( $pools ) == 0 ) {
                $list_node = $this->template->addFile( "projection_report_list.html" );
                $this->template->setDisplayDataImmediate( "parent", $this->projection->getId(), $list_node );
            } else {
                $this->template->addHeaderLink( "report.css" );
                $this->template->addFile( "projection_report_view.html" );
                $this->template->appendFileById( "projection_report_view_chart.html", "div", "report_chart" );
                $this->template->addFile( "instruct_report.html", "div" );

                $license = "";
                if ( I2CE::getConfig()->__isset( 'modules/maani-charts/license' ) ) {
                    $license = I2CE::getConfig()->__get( 'modules/maani-charts/license' );
                }
                $report_url = "report?data=1&" . $_SERVER{'QUERY_STRING'};
                if ( is_array( $includes ) ) {
                    $report_url .= "&check_includes=1";
                    foreach( $includes as $inc_id ) {
                        $report_url .= "&include[]=" . $inc_id;
                    }
                }
                $this->template->setDisplayDataImmediate( "charts", 
                        "file/charts.swf?license=" . $license
                        . "&library_path=file/charts_library"
                        . "&php_source=" . urlencode( $report_url ) );
                $this->template->setDisplayDataImmediate( "chart_cost", 
                        "file/charts.swf?license=" . $license
                        . "&library_path=file/charts_library"
                        . "&php_source=" . urlencode( $report_url . "&cost=1" ) );
    
                $population = $this->projection->projectPopulation();
                $title = array();
    
                $header_node = $this->template->appendFileByName( "projection_report_view_header.html", "th", "report_header" );
    
                $pop_row = $this->headerRow( "Population", "left_header" );
                foreach( $population['pop'] as $year => $pop ) {
                    $this->headerCell( $header_node, $year );
                    $this->dataCell( $pop_row, $pop, "bold", $population['real'][$year] );
                }
    
                $blank_span = count($population['pop'])+1;
                $this->blankRow( $blank_span );
    
                foreach( $pools as $pool ) {
                    $title[] = $pool->name;
                    $data = $pool->projectPool( $includes );
                    //print_r( $data );
    
                    $title_row = $this->headerRow( "Staff Category:", "left_header" );
                    $tmp_node = $this->template->appendFileByNode( "projection_report_view_data.html", "td", $title_row );
                    $this->template->setDisplayDataImmediate( "report_data", $pool->name, $tmp_node );
                    $tmp_node->setAttribute( "colspan", $blank_span-1 );
                    $tmp_node->setAttribute( "class", "title_row" );
    
                    $this->blankRow( $blank_span );
    
                    $target_row = $this->headerRow( "Required positions at start of year", "left_header" );
                    $tpp_row = $this->headerRow( "Target population number per position", "ital_header" );
    
                    $this->blankRow( $blank_span );
    
                    $begin_row = $this->headerRow( "Number of staff at start of year", "left_header" );
                    $bpp_row = $this->headerRow( "Population per employee at year start", "ital_header" );
    
                    $rows = array();
                    foreach( array_keys( $data['intake'] ) as $intake ) {
                        $class = "grey_header";
                        if ( is_array( $includes ) && !in_array( $intake, $includes ) ) {
                            $class = "grey_strike";
                        }

                        $rows["intake:" . $intake] = $this->headerRow( $intake . ":" . $data['intake'][$intake]['header'], $class, true );
                        if ( array_key_exists( $intake, $data['change_cost'] ) ) {
                            $rows["change_cost:" . $intake] = $this->headerRow( $data['change_cost'][$intake]['header'], $class );
                        }
                    }
                    $intake_row = $this->headerRow( "Total Intake", "grey_total" );
    
                    foreach( array_keys( $data['exit'] ) as $exit ) {
                        $class = "grey_header";
                        if ( is_array( $includes ) && !in_array( $exit, $includes ) ) {
                            $class = "grey_strike";
                        }
                        $rows["exit:" . $exit] = $this->headerRow( $exit . ":" . $data['exit'][$exit]['header'], $class, true );
                        if ( array_key_exists( $exit, $data['change_cost'] ) ) {
                            $rows["change_cost:" . $exit] = $this->headerRow( $data['change_cost'][$exit]['header'], $class );
                        }
                    }
                    $exit_row = $this->headerRow( "Total Exits", "grey_total" );
    
                    $end_row = $this->headerRow( "Actual staff at year end", "left_header" );

                    $sal_row = $this->headerRow( "Estimated Salary Cost", "left_header" );
    
                    $this->blankRow( $blank_span );
    
                    foreach( $population['pop'] as $year => $pop ) {
    
                        $total['target'][$year] += $data['target'][$year];
                        $total['begin_supply'][$year] += $data['begin_supply'][$year];
                        $total['end_supply'][$year] += $data['end_supply'][$year];
                        if ( $total['salary'][$year] instanceof iHRIS_FormField_CURRENCY ) {
                            $total['salary'][$year]->add( $data['salary'][$year] );
                            $total['cost'][$year]->add( $data['salary'][$year] );
                        } else {
                            $total['salary'][$year] = clone $data['salary'][$year];
                            $total['cost'][$year] = clone $data['salary'][$year];
                        }
    
                        $this->dataCell( $target_row, $data['target'][$year], "bold", $data['real']['target'][$year] );
                        $this->dataCell( $tpp_row, $data['target_per_pop'][$year], "bold", $data['real']['ratio'][$year] );
    
                        $this->dataCell( $begin_row, $data['begin_supply'][$year], "bold" . ($data['begin_supply'][$year] < $data['target'][$year] ? "_red" : "_green" ), $data['real']['begin_supply'][$year] );
                        $this->dataCell( $bpp_row, ($data['begin_supply'][$year] > 0 ? round( $pop / $data['begin_supply'][$year] ) : 0 ) );
    
                        foreach( array_keys( $data['intake'] ) as $intake ) {
                            $class = "grey";
                            if ( is_array( $includes ) && !in_array( $intake, $includes ) ) {
                                $class = "grey_strike";
                            } 

                            $this->dataCell( $rows["intake:" . $intake], $data['intake'][$intake][$year], $class );
                            if ( array_key_exists( "change_cost:" . $intake, $rows ) ) {
                                $this->costCell( $rows["change_cost:" . $intake], $data['change_cost'][$intake][$year], $class );
                                if ( $class == "grey" ) {
                                    $total['cost'][$year]->add( $data['change_cost'][$intake][$year] );
                                }
                            }
                        }
                        $this->dataCell( $intake_row, $data['intake_total'][$year], "grey" );
    
                        foreach( array_keys( $data['exit'] ) as $exit ) {
                            $class = "grey";
                            if ( is_array( $includes ) && !in_array( $exit, $includes ) ) {
                                $class = "grey_strike";
                            }
                            $this->dataCell( $rows["exit:" . $exit], $data['exit'][$exit][$year], $class );
                            if ( array_key_exists( "change_cost:" . $exit, $rows ) ) {
                                $this->costCell( $rows["change_cost:" . $exit], $data['change_cost'][$exit][$year], $class );
                                if ( $class == "grey" ) {
                                    $total['cost'][$year]->add( $data['change_cost'][$exit][$year] );
                                }
                            }
                        }
                        $this->dataCell( $exit_row, $data['exit_total'][$year], "grey" );
    
                        $this->dataCell( $end_row, $data['end_supply'][$year], "bold" );

                        $this->costCell( $sal_row, $data['salary'][$year], "bold" );
    
                    }
    
                }
    
                $this->blankRow( $blank_span );
    
                if ( count( $pools ) > 1 ) {
                    $title_row = $this->headerRow( "Staff Category:", "left_header" );
                    $tmp_node = $this->template->appendFileByNode( "projection_report_view_data.html", "td", $title_row );
                    $this->template->setDisplayDataImmediate( "report_data", "Total Staff", $tmp_node );
                    $tmp_node->setAttribute( "colspan", $blank_span-1 );
                    $tmp_node->setAttribute( "class", "title_row" );
        
                    $this->blankRow( $blank_span );
        
                    $target_row = $this->headerRow( "Required positions at start of year", "left_header" );
                    $tpp_row = $this->headerRow( "Target population number per position", "ital_header" );
        
                    $this->blankRow( $blank_span );
        
                    $begin_row = $this->headerRow( "Number of staff at start of year", "left_header" );
                    $bpp_row = $this->headerRow( "Population per employee at year start", "ital_header" );
                    $end_row = $this->headerRow( "Actual staff at year end", "left_header" );
                    $sal_row = $this->headerRow( "Estimated Total Salary Cost", "left_header" );
                    $this->blankRow( $blank_span );
                }
                $cost_row = $this->headerRow( "Total Cost", "left_header" );
                foreach( $population['pop'] as $year => $pop ) {
                    if ( count( $pools ) > 1 ) {
                        $this->dataCell( $target_row, $total['target'][$year], "bold" );
                        $this->dataCell( $tpp_row, ( $total['target'][$year] > 0 ? round( $pop / $total['target'][$year] ) : 0 ) );
        
                        $this->dataCell( $begin_row, $total['begin_supply'][$year], "bold" . ($total['begin_supply'][$year] < $total['target'][$year] ? "_red" : "_green" ) );
                        $this->dataCell( $bpp_row, ($total['begin_supply'][$year] > 0 ? round( $pop / $total['begin_supply'][$year] ) : 0 ) );
                        $this->dataCell( $end_row, $total['end_supply'][$year], "bold" );
                        $this->costCell( $sal_row, $total['salary'][$year], "bold" );
                    }
                    $this->costCell( $cost_row, $total['cost'][$year], "bold" );
                }
    
                $this->template->setDisplayData( "report_name", $this->projection->name . ": " . implode( ", ", $title ) );
            }
        }
        

    }

    /**
     * Add a header row template to the page with the given header and class.
     * @param string $header The text of the header.
     * @param string $class The CSS class for the row.
     * @param boolean $checkbox True if this header should have a checkbox to turn on and off
     * @return DOMNode
     */
    private function headerRow( $header, $class="", $checkbox=false ) {
        $header_row = $this->template->appendFileById( "projection_report_view_row.html", "tr", "report_body" );
        $this->headerCell( $header_row, $header, $class, $checkbox );
        return $header_row;
    }

    /**
     * Add a header cell template to the page with the given header and class.
     * @param DOMNode $row_node The node to append the header cell to.
     * @param string $header The text of the header.
     * @param string $class The CSS class for the cell.
     * @param boolean $checkbox True if this header should have a checkbox to turn on and off
     */
    private function headerCell( $row_node, $header, $class="", $checkbox=false ) {
        if ( $checkbox ) {
            $tmp_node = $this->template->appendFileByNode( "projection_report_view_checkbox.html", "th", $row_node );
            list( $id, $header ) = explode( ':', $header, 2 );
            $this->template->setDisplayDataImmediate( "include[]", $id, $tmp_node );
            $checked = true;
            if ( $this->post_exists( 'check_includes' ) ) {
                $includes = $this->post('include');
                if ( $includes == null ) $includes = array();
                if ( !in_array( $id, $includes ) ) {
                    $checked = false;
                }
            }
            if ( $checked ) {
                $result = $this->template->query( "input", $tmp_node );
                for( $i = 0; $i < $result->length; $i++ ) {
                    $result->item($i)->setAttribute( "checked", "checked" );
                }
            }
            $headers = explode( ' (', $header, 2 );
            if ( count( $headers ) == 2 ) {
                $this->template->setDisplayDataImmediate( "report_head", $headers[0], $tmp_node );
                $this->template->setDisplayDataImmediate( "report_sub", "(" . $headers[1], $tmp_node );
            } else {
                $this->template->setDisplayDataImmediate( "report_head", $header, $tmp_node );
            }
        } else {
            $tmp_node = $this->template->appendFileByNode( "projection_report_view_header.html", "th", $row_node );
            $this->template->setDisplayDataImmediate( "report_head", $header, $tmp_node );
        }
        if ( $class != "" ) $tmp_node->setAttribute( "class", $class );
    }

    /**
     * Add a data cell template to the page with the given value and class.
     * @param DOMNode $row_node The node to append the data cell to.
     * @param string $value The value data to display in this cell.
     * @param string $class The CSS class for the cell.
     * @param boolean $source_data Set to true if the data is from data entry instead of calculated.
     */
    private function dataCell( $row_node, $value, $class="", $source_data=false ) {
        $data_node = $this->template->appendFileByNode( "projection_report_view_data.html", "td", $row_node );
        $this->template->setDisplayDataImmediate( "report_data", ($source_data ? "*" : "") . number_format($value), $data_node );
        if ( $class != "" ) $data_node->setAttribute( "class", $class  . ($source_data ? "_italic" : "" ) );
    }

    /**
     * Add a data cell template to the page with the given salary value and class.
     * @param DOMNode $row_node The node to append the data cell to.
     * @param string $value The salary data to display in this cell.
     * @param string $class The CSS class for the cell.
     * @param boolean $source_data Set to true if the data is from data entry instead of calculated.
     */
    private function costCell( $row_node, $value, $class="", $source_data=false ) {
        $data_node = $this->template->appendFileByNode( "projection_report_view_data.html", "td", $row_node );
        if ( $value instanceof iHRIS_FormField_CURRENCY ) {
            $display_value = $value->getDisplayValue( false, true );
        } else {
            $display_value = 0;
        }
        $this->template->setDisplayDataImmediate( "report_data", ($source_data ? "*" : "") . $display_value, $data_node );
        if ( $class != "" ) $data_node->setAttribute( "class", $class  . ($source_data ? "_italic" : "" ) );
    }


    /**
     * Add a blank row to the page.
     * @param integer $width The colspan for the blank row.
     * @return DOMNode
     */
    private function blankRow( $width ) {
        $blank_row = $this->template->appendFileById( "projection_report_view_row.html", "tr", "report_body" );
        $tmp_node = $this->template->appendFileByNode( "projection_report_view_data.html", "td", $blank_row );
        $tmp_node->setAttribute( "colspan", $width );
        $tmp_node->setAttribute( "class", "blank_row" );
        return $blank_row;
    }

}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
