<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Search for a projection
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for searching for a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageSearch extends I2CE_Page {

    /**
     * @var I2CE_Form The search object for this page to display the form.
     */
    private $search;

    /**
     * Load the HTML template files for editing and confirming the index and demographic information.
     */
    protected function loadHTMLTemplates() {
        parent::loadHTMLTemplates();
        if ( $this->get_exists('do_search') ) {
            $this->template->addFile( "search_results.html" );
        } else {
            $this->template->addFile( "search_form.html" );
            $this->search->setFormLists( $this->template );
            $this->template->setForm( $this->search );
            $this->template->setShowForm();
        }
        $this->template->setAttribute( "class", "active", "menuProjection", "a[@href='main']" );
        $this->template->appendFileById( "menu_main.html", "ul", "menuProjection" );
        $this->template->setAttribute( "class", "active", "menuProjection", "ul/li/a[@href='search']" );
    }
        
    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        $factory = I2CE_FormFactory::instance();
        if ( $this->get('redirect') ) {
            header("Location: " . $this->get('redirect') );
        }
        $this->search = $factory->createForm( "search" );
        $this->search->load( $_GET );
        parent::action();
        if ( $this->get_exists('do_search') ) {
            $andor = "AND";
            $search = array();
            if ( $this->get('do_search') == 'find' && I2CE_Validate::checkString( $this->search->name ) ) {
                $search["projection"][] = array( 'field' => 'name', 
                                             'values' => array( 0 => array( 'value' => '%' . strtolower( $this->search->name ) . '%', 
                                                                            'method' => 'LIKE', 'lower' => true ) ) );
            }
            if ( I2CE_Validate::checkNumber( $this->search->country, 0 ) ) {
                $search['projection'][] = array( 'field' => 'country', 'values' => array( array( 'value' => $this->search->country ) ) );
            }

            $max_per_page = 100;
            if ( $this->get_exists('page') ) {
                $page = $this->get('page');
            } else {
                $page = 1;
            }            
            $results = $factory->callStatic( "projection", "search", array( $search, $andor, 
                array( (($page-1)*$max_per_page), $max_per_page ) ) );

            if ( array_key_exists( "count", $results ) ) {
                $result_count = $results["count"];
                unset( $results["count"] );
                $max_page = ceil( $result_count / $max_per_page );
                if ( $page < 1 || $page > $max_page ) {
                    $page = 1;
                }
            } else {
                $result_count = count($results);
            }

            if ( count( $results ) > 0 ) {
                $result_msg = $result_count . " records found.";
                if ( $result_count > count($results) ) {
                    $result_msg .= " " . count($results) . " displayed.";
                    $query = $this->search->getQueryFields();
                    $query['do_search'] = $this->get('do_search');
                    $this->template->appendElementById( "search_pager_display", "span", array(), "Go to page: ");
                    $this->makeJumper( "search", $page, $max_page, "search", $query );
                }
                $this->template->addText( '<div id="error">' . $result_msg . '</div>' );
                foreach( $results as $id => $info ) {
                    $display[ $info['category'] ][ $id ] = $info['name'];
                }
                ksort( $display );
                foreach( $display as $category => $projection ) {
                    $cat = $this->template->appendFileById( "search_category.html", "li", "list" );
                    $this->template->setDisplayDataImmediate( "category", $category, $cat );
                    $cat_ul = $this->template->query( "ul", $cat );
                    $this->template->setNodeAttribute( "id", "category_" . $category, $cat_ul->item(0) );
                    foreach( $projection as $id => $name ) {
                        $row = $this->template->appendFileById( "search_row.html", "li", "category_" . $category );
                        $this->template->setDisplayDataImmediate( "id", "id=" . $id, $row );
                        $this->template->setDisplayDataImmediate( "name", $name, $row );
                    }
                }
            } else {
                $this->template->addText( '<div id="error">No results found.</div>' );
            }
        }
    }
        
}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
