<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a cadre pool.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a cadre pool.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageViewPool extends I2CE_Page{ 
    /**
     * @var iHRIS_Pool The pool object associated with this page.
     */
    protected $pool;
        
    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        parent::action();
        $this->template->addHeaderLink("view.js");
        $this->template->addHeaderLink("missing.css");
        $this->template->appendFileById( "menu_view_link.html", "li", "navBarUL", true );
        $this->template->appendFileById( "menu_view_projection.html", "ul", "menuView" );
        $factory = I2CE_FormFactory::instance();
        $this->pool = $factory->createForm( "pool", $this->get('id') );
        $this->pool->populate();
        $projection = $factory->createForm( "projection", $this->pool->getParent() );
        $projection->populate();
        $projection->populateChildren( array( "pool" ) );
        foreach( $projection->children['pool'] as $pool ) {
            $node = $this->template->appendFileById( "menu_view_pool.html", "li", "menuViewProjection" );
            $this->template->setForm( $pool, $node );
            if ( $pool->getId() == $this->pool->getId() ) {
                $node->childNodes->item(1)->setAttribute( "class", "active" );
            }
        }
        $this->pool->populateChildren( array( "change" ) );
        $this->pool->populateLast( array( "supply" => "year", "target" => "year" ) );
        $this->template->setForm( $this->pool );
        $this->template->setForm( $projection );

        $seen = array();

        foreach( $this->pool->children as $form => $list ) {
            foreach( $list as $obj ) {
                if ( $obj instanceof I2CE_Form ) {
                    $seen[$form] = true;
                    $node = $this->template->appendFileById( "view_pool_" . $form . ".html", "div", $form );
                    $this->template->setForm( $obj, $node );
                }
            }
        }

        if ( !array_key_exists( 'supply', $seen ) || !$seen['supply'] ) {
            $this->template->appendFileById( "missing_supply.html", "div", "missing" );
        } elseif ( !array_key_exists( 'target', $seen ) || !$seen['target'] ) {
            $this->template->appendFileById( "missing_target.html", "div", "missing" );
        } elseif ( !array_key_exists( 'change', $seen ) || !$seen['change'] ) {
            $this->template->appendFileById( "missing_change.html", "div", "missing" );
        }

    }

}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
