<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * View a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The page class for displaying a projection.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_PageViewProjection extends I2CE_Page{ 
    protected $projection;
        
    /**
     * Perform the main actions of the page.
     */
    protected function action() {
        parent::action();
        $this->template->addHeaderLink("view.js");
        $this->template->addHeaderLink("missing.css");
        $this->template->appendFileById( "menu_view_link.html", "li", "navBarUL", true );
        $this->template->appendFileById( "menu_view_projection.html", "ul", "menuView" );
        $factory = I2CE_FormFactory::instance();
        $this->projection = $factory->createForm( "projection", $this->get('id') );
        $this->projection->populate();
        $this->projection->populateChildren( array( "pool" ) );
        $this->projection->populateLast( array( "population" => "year" ) );
        $this->template->setForm( $this->projection );

        $seen = array( 'supply' => true, 'target' => true, 'change' => true );

        foreach( $this->projection->children as $form => $list ) {
            foreach( $list as $obj ) {
                if ( $obj instanceof I2CE_Form ) {
                    $seen[$form] = true;
                    $node = $this->template->appendFileById( "view_" . $form . ".html", "div", $form );
                    $this->template->setForm( $obj, $node );
                    if ( $form == "pool" ) {
                        $obj->populateLast( array( "supply" => "year", "target" => "year" ) );
                        foreach( $obj->children as $pool_form => $pool_list ) {
                            foreach( $pool_list as $child ) {
                                $this->template->setForm( $child, $node );
                            }
                        }
                        if ( !array_key_exists( 'supply', $obj->children ) ) {
                            $seen['supply'] = false;
                        }
                        if ( !array_key_exists( 'target', $obj->children ) ) {
                            $seen['target'] = false;
                        }
                        $obj->populateChildren( array( "change" ) );
                        if ( !array_key_exists( 'change', $obj->children ) ) {
                            $seen['change'] = false;
                        }
                        $node = $this->template->appendFileById( "menu_view_pool.html", "li", "menuViewProjection" );
                        $this->template->setForm( $obj, $node );
                        //print_r( $obj->projectPool() );
                    }
                }
            }
        }

        if ( !array_key_exists( 'population', $seen ) || !$seen['population'] ) {
            $this->template->appendFileById( "missing_population.html", "div", "missing" );
        } elseif ( !array_key_exists( 'pool', $seen ) || !$seen['pool'] ) {
            $this->template->appendFileById( "missing_pool.html", "div", "missing" );
        } elseif ( !array_key_exists( 'supply', $seen ) || !$seen['supply'] ) {
            $this->template->appendFileById( "missing_cadre.html", "div", "missing" );
            foreach( $this->projection->children['pool'] as $obj ) {
                if ( !array_key_exists( 'supply', $obj->children ) ) {
                    $node = $this->template->appendFileById( "missing_cadre_supply.html", "p", "message" );
                    $this->template->setForm( $obj, $node );
                }
            }
         } elseif ( !array_key_exists( 'target', $seen ) || !$seen['target'] ) {
            $this->template->appendFileById( "missing_cadre.html", "div", "missing" );
            foreach( $this->projection->children['pool'] as $obj ) {
                if ( !array_key_exists( 'target', $obj->children ) ) {
                    $node = $this->template->appendFileById( "missing_cadre_target.html", "p", "message" );
                    $this->template->setForm( $obj, $node );
                }
            }
         } elseif ( !array_key_exists( 'change', $seen ) || !$seen['change'] ) {
            $this->template->appendFileById( "missing_cadre.html", "div", "missing" );
            foreach( $this->projection->children['pool'] as $obj ) {
                if ( !array_key_exists( 'change', $obj->children ) ) {
                    $node = $this->template->appendFileById( "missing_cadre_change.html", "p", "message" );
                    $this->template->setForm( $obj, $node );
                }
            }
        }

                
    }

}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
