<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS 
 * 
 * iHRIS is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * The cadre pool form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The cadre pool form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Pool extends I2CE_Form{

    /**
     * @var array The estimate pool data for this cadre pool.
     */
    private $__pool;

    /**
     * @var array The list of includes used to populate the current __pool
     */
    private $__last_includes;
    /**
     * @var boolean Flag to determine if the pool projection only includes the initial year's data.
     */
    private $__initial_year;

    /**
     * Check to make sure all the fields are valid.
     * @param I2CE_Template $page
     */
    public function validate( $page ) {
        parent::validate( $page );
        $factory = I2CE_FormFactory::instance();
        $parent_obj = $factory->createForm( "projection", $this->getParent() );
        $parent_obj->populate();
        $parent_obj->populateChildren( array( "pool" ) );
        if ( is_array( $parent_obj->children['pool'] ) ) {
            foreach( $parent_obj->children['pool'] as $pools ) {
                if ( $pools->getId() == $this->getId() ) continue;
                if ( $pools->name == $this->name ) {
                    $page->setFormInvalid( $this->getField( "name" )->getHTMLName(), 
                            "That pool name is already in use with this projection." );
                }
            }
        }
    }

    /**
     * Generate the projected data for this cadre pool.
     *
     * This method will calculate the future data for this cadre pool for the duration of the projection.
     * @param array $include_change A list of change forms to include in 
     * the projection totals.
     * @return array
     */
    public function projectPool( $include_change = null, $initial_year = false ) {
        if ( $this->__last_includes !== $include_change || $this->__initial_year !== $initial_year ||
                !is_array( $this->__pool ) || count( $this->__pool ) == 0 ) {
            $this->__last_includes = $include_change;
            $this->__initial_year = $initial_year;
            $factory = I2CE_FormFactory::instance();
            $proj = $factory->createForm( "projection", $this->getParent() );
            $this->populate();
            $this->populateChildren( array( "supply", "target", "change" ) );
            $real_data = array( 'supply' => array(), 'target' => array() );
            $change = array( 'intake' => array(), 'exit' => array() );
            foreach( $this->children as $form => $list ) {
                foreach( $list as $obj ) {
                    if ( $form == "change" ) {
                        if ( !$obj->enable ) continue;
                        $change_name = $obj->name;
                        if ( $obj->amount ) {
                            if ( $obj->rate ) {
                                $change_name .= " (" . $obj->amount . " +" . $obj->rate . "% per year";
                            } else {
                                $change_name .= " (" . $obj->amount . " per year";
                            }
                        } elseif ( $obj->rate ) {
                            $change_name .= " (" . $obj->rate . "% per year";
                        }
                        if ( $obj->year->year() > 0 ) {
                            $change_name .= " starting " . $obj->year->year();
                            if ( $obj->duration > 0 ) {
                                $change_name .= " for " . $obj->duration . " years";
                            }
                        } else {
                            $change_name .= " for every year";
                        }
                        $change_name .= ")";
                        $change[ ($obj->intake == iHRIS_Change::CHANGE_INTAKE ? 'intake' : 'exit') ][ $obj->getId() ] 
                            = array( "name" => $change_name, 
                                    "amount" => $obj->amount, "rate" => $obj->rate,
                                    "year" => $obj->year->year(), "duration" => $obj->duration,
                                    "avg_cost" => $obj->getField( "avg_cost" ), "avg_increase" => $obj->avg_increase,
                                    "cost_header" => $obj->name . " Cost" );
                    } elseif ( $form == "supply" ) {
                        $real_data[$form][ $obj->year->year() ] = array( "supply" => $obj->supply, 
                                "avg_salary" => $obj->getField("avg_salary"), "avg_increase" => $obj->avg_increase );

                    } elseif ( $form == "target" ) {
                        $real_data[$form][ $obj->year->year() ] = array( "target" => $obj->target,
                                "ratio" => $obj->ratio );
                    } else {
                        $real_data[$form][ $obj->year->year() ] = $obj->$form;
                    }
                }
            }
            $this->__pool = array(
                    'target' => array(), 'target_per_pop' => array(),
                    'intake_total' => array(), 'exit_total' => array(), 'begin_supply' => array(), 'end_supply' => array(),
                    'intake' => array(), 'exit' => array(), 'change_cost' => array(), 'total_cost' => array(),
                    );
            $population = $proj->projectPopulation( $initial_year );
            $target_per_pop = 0;
            $next_supply = 0;
            $next_salary = new iHRIS_FormField_CURRENCY( "avg_salary", array( "in_db" => false ) );
            $prev_change_amt = array();
            $salary_increase = 0;
            $next_cost = array();
            foreach( $population['pop'] as $year => $pop ) {
                //$this->__pool['population'][ $year ] = $pop;
                if ( ( $initial_year == false || $year == $initial_year ) 
                        && array_key_exists( $year, $real_data['target'] ) ) {
                    if ( $real_data['target'][$year]['target'] ) {
                        $this->__pool['target'][ $year ] = $real_data['target'][$year]['target'];
                        $this->__pool['real']['target'][$year] = true;
                        $target_per_pop = round( $pop / $this->__pool[ 'target' ][ $year ] );
                    } elseif( $real_data['target'][$year]['ratio'] ) {
                        $target_per_pop = $real_data['target'][$year]['ratio'];
                        $this->__pool['target'][ $year ] = round( $pop / $target_per_pop );
                        $this->__pool['real']['ratio'][$year] = true;
                    } else {
                        $this->__pool['target'][ $year ] = 0;
                    }
                } elseif( $target_per_pop > 0 ) {
                    $this->__pool['target'][ $year ] = round( $pop / $target_per_pop );
                } else {
                    $this->__pool['target'][ $year ] = 0;
                }
                $this->__pool[ 'target_per_pop' ][ $year ] = $target_per_pop;
                if ( ( $initial_year == false || $year == $initial_year ) 
                        && array_key_exists( $year, $real_data['supply'] ) ) {
                    $curr_supply = $real_data['supply'][$year]['supply'];
                    $this->__pool['real']['begin_supply'][$year] = true;
                    $curr_salary = clone $real_data['supply'][$year]['avg_salary'];
                    $next_salary = clone $curr_salary;
                    $salary_increase = $real_data['supply'][$year]['avg_increase'];
                } else {
                    $curr_supply = $next_supply;
                    $curr_salary = clone $next_salary;
                }
                //$next_salary[1] = round( $curr_salary[1] + ( $curr_salary[1] * $salary_increase / 100 ) );
                $next_salary->multiply( (100 + $salary_increase) / 100 );
                $this->__pool['begin_supply'][ $year ] = $curr_supply;

                $this->__pool['intake_total'][ $year ] = 0;
                if ( array_key_exists( 'intake', $change ) ) {
                    foreach( $change['intake'] as $id => $change_arr ) {
                        if ( $change_arr['year'] > 0 ) {
                            if ( $year < $change_arr['year'] ) continue;
                            if ( $change_arr['duration'] && $change_arr['duration'] > 0 
                                    && $year >= $change_arr['year']+$change_arr['duration'] ) continue;
                        }
                        if ( $change_arr['amount'] ) {
                            if ( array_key_exists( $id, $prev_change_amt ) && $change_arr['rate'] > 0 ) {
                                $change_amt = $prev_change_amt[$id] 
                                    + round( $prev_change_amt[$id] * $change_arr['rate'] / 100 );
                                $prev_change_amt[$id] = $change_amt;
                            } else {
                                $change_amt = $change_arr['amount'];
                                $prev_change_amt[$id] = $change_amt;
                            }
                        } elseif ( $change_arr['rate'] ) {
                            $change_amt = round( $curr_supply * $change_arr['rate'] / 100 );
                        }
                        $this->__pool['intake'][$id]['header'] = $change_arr['name'];
                        $this->__pool['intake'][$id][ $year ] = $change_amt;

                        if ( $change_arr['avg_cost'] instanceof iHRIS_FormField_CURRENCY 
                                && $change_arr['avg_cost']->isValid() ) {
                            if ( array_key_exists( $id, $next_cost ) ) {
                                $next_cost[$id]->multiply( ( 100 + $change_arr['avg_increase'] ) / 100 );
                            } else {
                                $next_cost[$id] = clone $change_arr['avg_cost'];
                            }
                            $this->__pool['change_cost'][$id][$year] = clone $next_cost[$id];
                            $this->__pool['change_cost'][$id][$year]->multiply( $change_amt );
                            $this->__pool['change_cost'][$id]['header'] = $change_arr['cost_header'];
                            if ( !is_array( $include_change ) || in_array( $id, $include_change ) ) {
                                if ( array_key_exists( $year, $this->__pool['total_cost'] ) ) {
                                    $this->__pool['total_cost'][$year]->add( $this->__pool['change_cost'][$id][$year] );
                                } else {
                                    $this->__pool['total_cost'][$year] = clone $this->__pool['change_cost'][$id][$year];
                                }
                            }
                        }

                        if ( !is_array( $include_change ) || in_array( $id, $include_change ) ) {
                            $this->__pool['intake_total'][ $year ] += $change_amt;
                        }
                    }
                }
                $this->__pool['exit_total'][ $year ] = 0;
                if ( array_key_exists( 'exit', $change ) ) {
                    foreach( $change['exit'] as $id => $change_arr ) {
                         if ( $change_arr['year'] > 0 ) {
                            if ( $year < $change_arr['year'] ) continue;
                            if ( $change_arr['duration'] && $change_arr['duration'] > 0 
                                    && $year >= $change_arr['year']+$change_arr['duration'] ) continue;
                        }
                        if ( $change_arr['amount'] ) {
                            if ( array_key_exists( $id, $prev_change_amt ) && $change_arr['rate'] > 0 ) {
                                $change_amt = $prev_change_amt[$id] 
                                    + round( $prev_change_amt[$id] * $change_arr['rate'] / 100 );
                                $prev_change_amt[$id] = $change_amt;
                            } else {
                                $change_amt = $change_arr['amount'];
                                $prev_change_amt[$id] = $change_amt;
                            }
                        } elseif ( $change_arr['rate'] ) {
                            $change_amt = round( $curr_supply * $change_arr['rate'] / 100 );
                        }
                        $this->__pool['exit'][$id][ 'header' ] = $change_arr['name'];
                        $this->__pool['exit'][$id][ $year ] = $change_amt;

                        if ( $change_arr['avg_cost'] instanceof iHRIS_FormField_CURRENCY 
                                && $change_arr['avg_cost']->isValid() ) {
                            if ( array_key_exists( $id, $next_cost ) ) {
                                $next_cost[$id]->multiply( ( 100 + $change_arr['avg_increase'] ) / 100 );
                            } else {
                                $next_cost[$id] = clone $change_arr['avg_cost'];
                            }
                            $this->__pool['change_cost'][$id][$year] = clone $next_cost[$id];
                            $this->__pool['change_cost'][$id][$year]->multiply( $change_amt );
                            $this->__pool['change_cost'][$id]['header'] = $change_arr['cost_header'];
                            if ( !is_array( $include_change ) || in_array( $id, $include_change ) ) {
                                if ( array_key_exists( $year, $this->__pool['total_cost'] ) ) {
                                    $this->__pool['total_cost'][$year]->add( $this->__pool['change_cost'][$id][$year] );
                                } else {
                                    $this->__pool['total_cost'][$year] = clone $this->__pool['change_cost'][$id][$year];
                                }
                            }
                        }

                        if ( !is_array( $include_change ) || in_array( $id, $include_change ) ) {
                            $this->__pool['exit_total'][ $year ] += $change_amt;
                        }
                    }
                }
                $next_supply = $curr_supply + $this->__pool['intake_total'][ $year ] - $this->__pool['exit_total'][ $year ];
                $this->__pool['end_supply'][ $year ] = $next_supply;
                $curr_salary->multiply( ($curr_supply + $next_supply) / 2 );
                $this->__pool['salary'][ $year ] = $curr_salary;
                if ( array_key_exists( $year, $this->__pool['total_cost'] ) ) {
                    $this->__pool['total_cost'][$year]->add( $this->__pool['salary'][$year] );
                } else {
                    $this->__pool['total_cost'][$year] = clone $this->__pool['salary'][$year];
                }
            }
        }
        return $this->__pool;
    }


}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
