<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS 
 * 
 * iHRIS is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * The target form.
 * @package iHRIS
 * @subpackage Plan
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @access public
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The target form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Target extends I2CE_Form{


    /**
     * Create a new instance of a target form.
     * 
     * @param string $form The name of this form
     * @param integer $id
     */
    public function __construct( $form, $id = 0 ) {                
        parent::__construct( $form, $id );
        $this->year = I2CE_Date::now( I2CE_Date::YEAR_ONLY );
    }

    /**
     * Check to make sure all the fields are valid.
     * @param I2CE_Template $page
     */
    public function validate( $page ) {
        parent::validate( $page );
        $factory = I2CE_FormFactory::instance();
        $parent_obj = $factory->createForm( "pool", $this->getParent() );
        $parent_obj->populate();
        $proj = $factory->createForm( "projection", $parent_obj->getParent() );
        $proj->populate();
        $start_year = $proj->start_year->year();
        $duration = $proj->duration;
        $year = $this->year->year();
        if ( !I2CE_Validate::checkNumber( $this->target, 0 ) && !I2CE_Validate::checkNumber( $this->ratio, 0 ) ) {
            $page->setFormInvalid( $this->getField( "ratio" )->getHTMLName(), "Please enter either a target number or ratio." );
            $page->setFormInvalid( $this->getField( "target" )->getHTMLName(), "Please enter either a target number or ratio." );
        } elseif ( I2CE_Validate::checkNumber( $this->target, 0 ) && I2CE_Validate::checkNumber( $this->ratio, 0 ) ) {
            $page->setFormInvalid( $this->getField( "ratio" )->getHTMLName(), "Please enter either a target number OR ratio, not both." );
            $page->setFormInvalid( $this->getField( "target" )->getHTMLName(), "Please enter either a target number OR ratio, not both." );
        }
        if ( $year < $proj->start_year->year() || $year > $start_year+$duration  ) {
            $page->setFormInvalid( $this->getField( "year" )->getHTMLName(), "The year is outside the range for this projection." );
        } else {
            $parent_obj->populateChildren( array( "target" ) );
            if ( is_array( $parent_obj->children['target'] ) ) {
                foreach( $parent_obj->children['target'] as $target ) {
                    if ( $target->getId() == $this->getId() ) continue;
                    if ( $target->year->year() == $year ) {
                        $page->setFormInvalid( $this->getField( "year" )->getHTMLName(), 
                                "That year already has target data saved.  Please modify that information instead of adding a new record." );
                    }
                }
            }
        }
    }


}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
